package ms.vmedinap.tfg.jwegenerator.configuration;

import com.nimbusds.jose.crypto.RSAEncrypter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;
import org.springframework.util.ResourceUtils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.security.*;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@Configuration
@EnableAutoConfiguration
public class KeysConfiguration {

    @Value("${jwe.certificate.filename}")
    private String certificateName;

    @Value("${jwe.certificate.alias}")
    private String certificateAlias;

    @Value("${jwe.certificate.pwd}")
    private String certificatePwd;

    @Value("${jwe.signingkey.filename}")
    private String signingKeyName;

    @Value("${jwe.verifierkey.filename}")
    private String verifierKeyName;

    @Bean(name = "signingKey")
    public PrivateKey getSigningKey() throws IOException, NoSuchAlgorithmException, InvalidKeySpecException {
        InputStream stream = new ClassPathResource(signingKeyName).getInputStream();
        byte[] keyBytes = stream.readAllBytes();
        PKCS8EncodedKeySpec spec =
                new PKCS8EncodedKeySpec(keyBytes);
        KeyFactory kf = KeyFactory.getInstance("RSA");
        return kf.generatePrivate(spec);
    }

    @Bean(name = "rsaSSAVerifier")
    public RSAPublicKey getRSASSAVerifier() throws IOException, NoSuchAlgorithmException, InvalidKeySpecException {
        InputStream stream = new ClassPathResource(verifierKeyName).getInputStream();
        byte[] keyBytes = stream.readAllBytes();

        X509EncodedKeySpec spec =
                new X509EncodedKeySpec(keyBytes);
        KeyFactory kf = KeyFactory.getInstance("RSA");
        return (RSAPublicKey) kf.generatePublic(spec);
    }

    @Bean(name = "rsaEncrypter")
    public RSAEncrypter getRSAEncrypter() throws IOException, KeyStoreException, CertificateException,
            NoSuchAlgorithmException {
        KeyStore ks;
        X509Certificate x509Certificate;
        InputStream stream = new ClassPathResource(certificateName).getInputStream();
        byte[] pfxCertificate = stream.readAllBytes();
        try (InputStream inputStream = new ByteArrayInputStream(pfxCertificate)) {
            ks = KeyStore.getInstance("PKCS12");
            ks.load(inputStream, certificatePwd.toCharArray());
            x509Certificate = (X509Certificate) ks.getCertificate(certificateAlias);
        }
        return new RSAEncrypter((RSAPublicKey) x509Certificate.getPublicKey());
    }

    @Bean(name = "rsaDecrypter")
    public PrivateKey getRSADecrypter() throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException,
            CertificateException, IOException {
        KeyStore ks;
        InputStream stream = new ClassPathResource(certificateName).getInputStream();
        byte[] pfxCertificate = stream.readAllBytes();
        try (InputStream inputStream = new ByteArrayInputStream(pfxCertificate)) {
            ks = KeyStore.getInstance("PKCS12");
            ks.load(inputStream, certificatePwd.toCharArray());
        }
        return (PrivateKey) ks.getKey(certificateAlias, certificatePwd.toCharArray());
    }
}
