package ms.vmedinap.tfg.jwegenerator.services.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nimbusds.jose.*;
import com.nimbusds.jose.crypto.RSADecrypter;
import com.nimbusds.jose.crypto.RSAEncrypter;
import com.nimbusds.jose.crypto.RSASSASigner;
import com.nimbusds.jose.crypto.RSASSAVerifier;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.SignedJWT;
import ms.vmedinap.tfg.jwegenerator.models.request.JWERequest;
import ms.vmedinap.tfg.jwegenerator.services.JweService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.security.PrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.text.ParseException;

/**
 * Trabajo: TFG
 * Título: Seguridad adicional frente a ciber ataques “Man-In-The-Middle”
 * Carrera: Grado en Ingeniería Informática
 * Área: Ingeniería del Software
 * Curso: 2021-2022
 *
 * @author Víctor Medina Pérez
 */
@Service
public class JweServiceImpl implements JweService {

    @Qualifier("signingKey")
    private final PrivateKey signingKey;

    @Qualifier("rsaSSAVerifier")
    private final RSAPublicKey rsaSSAVerifier;

    @Qualifier("rsaEncrypter")
    private final RSAEncrypter rsaEncrypter;

    @Qualifier("rsaDecrypter")
    private final PrivateKey rsaDecrypter;

    @Autowired
    public JweServiceImpl(PrivateKey signingKey, RSAPublicKey rsaSSAVerifier,
                          RSAEncrypter rsaEncrypter, PrivateKey rsaDecrypter) {
        this.signingKey = signingKey;
        this.rsaSSAVerifier = rsaSSAVerifier;
        this.rsaEncrypter = rsaEncrypter;
        this.rsaDecrypter = rsaDecrypter;
    }

    @Override
    public JWEObject decryptJWE(String token) throws ParseException, JOSEException {
        JWEObject jweObject;
        jweObject = JWEObject.parse(token);
        jweObject.decrypt(new RSADecrypter(rsaDecrypter));
        return jweObject;
    }

    @Override
    public boolean jwsValidation(SignedJWT signedJWT) throws JOSEException {
        return signedJWT.verify(new RSASSAVerifier(rsaSSAVerifier));
    }

    @Override
    public ResponseEntity<String> generateJWE(JWERequest jweRequest) throws IOException, ParseException,
            JOSEException {

        ObjectMapper objectMapper = new ObjectMapper();

        String claims = objectMapper.writeValueAsString(jweRequest);

        JWTClaimsSet jwtClaimsSet = JWTClaimsSet.parse(claims);

        JWSSigner signer = new RSASSASigner(signingKey);

        SignedJWT signedJWT = new SignedJWT(
                new JWSHeader.Builder(JWSAlgorithm.RS256)
                        .keyID("publicKey")
                        .contentType("JWT")
                        .type(JOSEObjectType.JWT)
                        .build(),
                jwtClaimsSet
        );

        // Sign the JWT
        signedJWT.sign(signer);

        // On the consumer side, parse the JWS and verify its RSA signature
        signedJWT = SignedJWT.parse(signedJWT.serialize());

        JWEObject jweObject = new JWEObject(
                new JWEHeader.Builder(JWEAlgorithm.RSA_OAEP_256, EncryptionMethod.A256GCM)
                        .contentType("JWE")
                        .keyID("jweCertificate")
                        .build(),
                new Payload(signedJWT)
        );

        jweObject.encrypt(rsaEncrypter);

        return new ResponseEntity<>(jweObject.serialize(), HttpStatus.OK);
    }
}
